@@ -63,3 +63,19 @@ class TestModel(ModelBaseTest):
         data = {'age': 1, 'name': 'foo'}
         model = model_class(data)
         self.assertEqual(model.uuid, 'foo')
+
+    def test_update(self):
+        model_class = self.mk_model({
+            'uuid': 'foo',
+            'age': IntegerField('An age'),
+            'name': TextField('A name'),
+        })
+        data = {'age': 1, 'name': 'foo'}
+        model = model_class(data)
+        self.assertFalse(model.is_read_only())
+        new_model = model.update({'age': 20})
+        self.assertEqual(new_model.age, 20)
+        self.assertEqual(new_model.uuid, model.uuid)
+        self.assertEqual(new_model.name, model.name)
+        self.assertTrue(model.is_read_only())
+        self.assertFalse(new_model.is_read_only())
